local addonName, addon = ...

-- functions
local format = string.format
local unpack, wipe = unpack, wipe
local IsControlKeyDown = IsControlKeyDown
local IsShiftKeyDown = IsShiftKeyDown
-- constants
-- frames
local GameTooltip = GameTooltip
-- libs
local LibStub = LibStub

local broker = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject(addonName, {
    type = "data source",
    label = "Keystone",
    text = "No Keystone",
    icon = [[Interface\Icons\inv_relics_hourglass]],
})

local function GetTooltipLinesForPool(self, pool)
    if not pool then return {} end

    local lines = {}

    for key, character in pairs(pool) do
        if character.keystone then
            table.insert(lines, {
                self:GetCharacterDisplayName(key, true),
                addon:GetKeystoneDisplayText(character.keystone),
            })
        end
    end

    return lines
end

local function GetTooltipLines(self)
    local lines = {}
    if self.debug then
        table.insert(lines, {"Debug mode enabled", nil, 1, 0, 0})
    end

    local pool, poolLines = {}, {}

    -- our own keystones
    pool = self.db.pools["player"]
    if pool then
        wipe(poolLines)
        poolLines = GetTooltipLinesForPool(self, pool)
        if #poolLines > 0 then
            if #lines > 0 then table.insert(lines, " ") end
            table.insert(lines, {
                "Character",
                format("%d |cffFFFFFF%s|r", #poolLines, (#poolLines == 1 and "Keystone" or "Keystones")),
                1.0, 1.0, 1.0,
            })
            table.insert(lines, " ")
            for i = 0, #poolLines do
                table.insert(lines, poolLines[i])
            end
        end
    end

    if #lines < 1 then
        table.insert(lines, {"No Keystones", nil, 1.0, 1.0, 1.0})
        return lines
    end

    return lines
end

function broker:OnTooltipShow()
    local lines = GetTooltipLines(addon)

    for _, line in pairs(lines) do
        if type(line) == "string" then
            line = {line}
        end

        self:AddDoubleLine(unpack(line))
    end
end

function broker:OnEnter()
    GameTooltip:SetOwner(self, "ANCHOR_NONE")
    GameTooltip:SetPoint("BOTTOM", self, "BOTTOM")
    GameTooltip:ClearLines()
    broker.OnTooltipShow(GameTooltip)
    GameTooltip:Show()
end

function broker:OnLeave()
    GameTooltip:Hide()
end

function broker:OnClick(btn)
    if btn == "LeftButton" then return broker:OnLeftClick() end
    if btn == "RightButton" then return broker:OnRightClick() end
end

function broker:OnLeftClick()
    if IsShiftKeyDown() then
        if addon.player.keystone ~= nil then
            ChatFrame_OpenChat(addon.player.keystone)
        else
            print("You don't have a keystone on this character.")
            print("If you did, shift-clicking the broker would link it to your chat.")
        end
    end
end

broker.menuFrame = CreateFrame("frame", "BrokerKeystonesContextMenuFrame", UIParent, "UIDropDownMenuTemplate")
function broker:OnRightClick()
    EasyMenu({
        {text = addonName, isTitle = true},
        {text = "Use shorthand dungeon names where available", checked = addon.options.useShorthand, func = function()
            addon:ToggleAbbreviatedDungeonNames()
        end},
        {text = "Delete Keystone Data", func = function()
            addon:DeleteAllCharacterKeystones()
        end},
    }, broker.menuFrame, "cursor", 0, 0, "MENU")
end

addon:RegisterCallback(addon.EVENT_KEYSTONE, function(_, key, link)
    broker.text = link and addon:GetKeystoneDisplayText(link) or "No Keystone"
end)
